==> firefox-65.0b9.source.tar.xz: firefox-65.0/dom/animation/DocumentTimeline.h <==
/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set ts=8 sts=2 et sw=2 tw=80: */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

#ifndef mozilla_dom_DocumentTimeline_h
#define mozilla_dom_DocumentTimeline_h

#include "mozilla/dom/DocumentTimelineBinding.h"
#include "mozilla/LinkedList.h"
#include "mozilla/TimeStamp.h"
#include "AnimationTimeline.h"
#include "nsIDocument.h"
#include "nsDOMNavigationTiming.h"  // for DOMHighResTimeStamp
#include "nsRefreshDriver.h"

struct JSContext;

namespace mozilla {
namespace dom {

class DocumentTimeline final : public AnimationTimeline,
                               public nsARefreshObserver,
                               public nsATimerAdjustmentObserver,
                               public LinkedListElement<DocumentTimeline> {
 public:
  DocumentTimeline(nsIDocument* aDocument, const TimeDuration& aOriginTime)
      : AnimationTimeline(aDocument->GetParentObject()),
        mDocument(aDocument),
        mIsObservingRefreshDriver(false),
        mOriginTime(aOriginTime) {
    if (mDocument) {
      mDocument->Timelines().insertBack(this);
    }
  }

 protected:
  virtual ~DocumentTimeline() {
    MOZ_ASSERT(!mIsObservingRefreshDriver,
               "Timeline should have disassociated"
               " from the refresh driver before being destroyed");
    if (isInList()) {
      remove();
    }
  }

 public:
  NS_DECL_ISUPPORTS_INHERITED
  NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS_INHERITED(DocumentTimeline,
                                                         AnimationTimeline)

  virtual JSObject* WrapObject(JSContext* aCx,
                               JS::Handle<JSObject*> aGivenProto) override;

  static already_AddRefed<DocumentTimeline> Constructor(
      const GlobalObject& aGlobal, const DocumentTimelineOptions& aOptions,
      ErrorResult& aRv);

  // AnimationTimeline methods

  // This is deliberately _not_ called GetCurrentTime since that would clash
  // with a macro defined in winbase.h
  virtual Nullable<TimeDuration> GetCurrentTimeAsDuration() const override;

  bool TracksWallclockTime() const override {
    nsRefreshDriver* refreshDriver = GetRefreshDriver();
    return !refreshDriver ||
           !refreshDriver->IsTestControllingRefreshesEnabled();
  }
  Nullable<TimeDuration> ToTimelineTime(
      const TimeStamp& aTimeStamp) const override;
  TimeStamp ToTimeStamp(const TimeDuration& aTimelineTime) const override;

  void NotifyAnimationUpdated(Animation& aAnimation) override;

  void RemoveAnimation(Animation* aAnimation) override;

  // nsARefreshObserver methods
  void WillRefresh(TimeStamp aTime) override;
  // nsATimerAdjustmentObserver methods
  void NotifyTimerAdjusted(TimeStamp aTime) override;

  void NotifyRefreshDriverCreated(nsRefreshDriver* aDriver);
  void NotifyRefreshDriverDestroying(nsRefreshDriver* aDriver);

  nsIDocument* GetDocument() const override { return mDocument; }

 protected:
  TimeStamp GetCurrentTimeStamp() const;
  nsRefreshDriver* GetRefreshDriver() const;
  void UnregisterFromRefreshDriver();
  void MostRecentRefreshTimeUpdated();
  void ObserveRefreshDriver(nsRefreshDriver* aDriver);
  void DisconnectRefreshDriver(nsRefreshDriver* aDriver);

  nsCOMPtr<nsIDocument> mDocument;

  // The most recently used refresh driver time. This is used in cases where
  // we don't have a refresh driver (e.g. because we are in a display:none
  // iframe).
  mutable TimeStamp mLastRefreshDriverTime;
  bool mIsObservingRefreshDriver;

  TimeDuration mOriginTime;
};

}  // namespace dom
}  // namespace mozilla

#endif  // mozilla_dom_DocumentTimeline_h

==> firefox-84.0.2.source.tar.xz: firefox-84.0.2/dom/animation/DocumentTimeline.h <==
/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set ts=8 sts=2 et sw=2 tw=80: */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

#ifndef mozilla_dom_DocumentTimeline_h
#define mozilla_dom_DocumentTimeline_h

#include "mozilla/dom/Document.h"
#include "mozilla/dom/DocumentTimelineBinding.h"
#include "mozilla/LinkedList.h"
#include "mozilla/TimeStamp.h"
#include "AnimationTimeline.h"
#include "nsDOMNavigationTiming.h"  // for DOMHighResTimeStamp
#include "nsRefreshDriver.h"
#include "nsRefreshObservers.h"

struct JSContext;

namespace mozilla {
namespace dom {

class DocumentTimeline final : public AnimationTimeline,
                               public nsARefreshObserver,
                               public nsATimerAdjustmentObserver,
                               public LinkedListElement<DocumentTimeline> {
 public:
  DocumentTimeline(Document* aDocument, const TimeDuration& aOriginTime)
      : AnimationTimeline(aDocument->GetParentObject()),
        mDocument(aDocument),
        mIsObservingRefreshDriver(false),
        mOriginTime(aOriginTime) {
    if (mDocument) {
      mDocument->Timelines().insertBack(this);
    }
  }

 protected:
  virtual ~DocumentTimeline() {
    MOZ_ASSERT(!mIsObservingRefreshDriver,
               "Timeline should have disassociated"
               " from the refresh driver before being destroyed");
    if (isInList()) {
      remove();
    }
  }

 public:
  NS_DECL_ISUPPORTS_INHERITED
  NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS_INHERITED(DocumentTimeline,
                                                         AnimationTimeline)

  virtual JSObject* WrapObject(JSContext* aCx,
                               JS::Handle<JSObject*> aGivenProto) override;

  static already_AddRefed<DocumentTimeline> Constructor(
      const GlobalObject& aGlobal, const DocumentTimelineOptions& aOptions,
      ErrorResult& aRv);

  // AnimationTimeline methods

  // This is deliberately _not_ called GetCurrentTime since that would clash
  // with a macro defined in winbase.h
  virtual Nullable<TimeDuration> GetCurrentTimeAsDuration() const override;

  bool TracksWallclockTime() const override;
  Nullable<TimeDuration> ToTimelineTime(
      const TimeStamp& aTimeStamp) const override;
  TimeStamp ToTimeStamp(const TimeDuration& aTimelineTime) const override;

  void NotifyAnimationUpdated(Animation& aAnimation) override;

  void RemoveAnimation(Animation* aAnimation) override;

  // nsARefreshObserver methods
  void WillRefresh(TimeStamp aTime) override;
  // nsATimerAdjustmentObserver methods
  void NotifyTimerAdjusted(TimeStamp aTime) override;

  void NotifyRefreshDriverCreated(nsRefreshDriver* aDriver);
  void NotifyRefreshDriverDestroying(nsRefreshDriver* aDriver);

  Document* GetDocument() const override { return mDocument; }

 protected:
  TimeStamp GetCurrentTimeStamp() const;
  nsRefreshDriver* GetRefreshDriver() const;
  void UnregisterFromRefreshDriver();
  void MostRecentRefreshTimeUpdated();
  void ObserveRefreshDriver(nsRefreshDriver* aDriver);
  void DisconnectRefreshDriver(nsRefreshDriver* aDriver);

  RefPtr<Document> mDocument;

  // The most recently used refresh driver time. This is used in cases where
  // we don't have a refresh driver (e.g. because we are in a display:none
  // iframe).
  mutable TimeStamp mLastRefreshDriverTime;
  bool mIsObservingRefreshDriver;

  TimeDuration mOriginTime;
};

}  // namespace dom
}  // namespace mozilla

#endif  // mozilla_dom_DocumentTimeline_h
